/*****************************************************************************
 * $Id: game_pony_defs_eng.hh,v 1.4 2005/10/26 18:27:51 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Definitions, constants, settings (some which engine depends on).
 * @remarks @verbatim
 * ----------------------------------------------------------------------------
 * This header file should only contain self-contained constants (no dependencies).
 * C++ const is preferred to use type-checking but some must be #defined
 * as some source files test them using #if.
 * NOTE: Be sure to WRITE A PERIOD IN FLOAT CONSTANTS defined by #define
 * or integer instructions could be compiled (with loss of accuracy).
 * ----------------------------------------------------------------------------
 * @endverbatim
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#ifndef GAME_PONY_ENG_DEFS_HH
#define GAME_PONY_ENG_DEFS_HH 1

//==============================================================================
//						The engine depends on these definitions
//------------------------------------------------------------------------------
/// @def WORLD
/// All local and world coordinate constants should be wrapped by
/// WORLD()/LOCAL() in case in the future coordinate systems need to be scaled.
/// Should be applied only to literal constants (not computed values nor macros).
#define	LOCAL(coord) (coord)
#define	WORLD(coord) (coord)
const fp	MAX_ALTITUDE_DETAIL			= WORLD(7000.0);	///< eg trees are rendered below this altitude
const fp	INC_TRANSLATE				= WORLD(5.0/2.0);
const fp	INC_TRANSLATE_EYE			= WORLD(6.0/2.0);
const fp	INC_TRANSLATE_CRAFT			= WORLD(4.0/2.0);
const fp	INC_ROTATE					= 2;
const fp	INC_ROTATE_EYE				= 1.0;
const fp	INC_ROTATE_CRAFT			= 1.5;
const fp	INC_TRACER					= WORLD(50.0);
const fp	DYNA_MIN_ALTITUDE			= WORLD(20.0);		///< crashed if below
const fp	EYE_MIN_ALTITUDE			= WORLD(20.0);		///< crashed if below
//==============================================================================

#endif // GAME_PONY_ENG_DEFS_HH
